from django.core.management.base import BaseCommand
from django.utils import timezone


class Command(BaseCommand):
    help = "Enable/Disabled Maintenance Mode if time is passed"

    def handle(self, *args, **options):
        from django_maintenance_window.models import MaintenanceMode
        config = MaintenanceMode.get_solo()
        now = timezone.now()

        if config.maintenance:
            if config.maintenance_until < now:
                config.maintenance = False
                config.maintenance_until = None
                config.save()
        else:
            if config.maintenance_from and config.maintenance_from < now:
                config.maintenance = True
                config.maintenance_from = None
                config.save()



# -*- coding: utf-8 -*-

import django
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils.cache import add_never_cache_headers

from . import settings
from .models import MaintenanceMode

if django.VERSION < (1, 10):
    MaintenanceModeMiddlewareBaseClass = object
else:
    # https://docs.djangoproject.com/en/1.10/topics/http/middleware/#upgrading-pre-django-1-10-style-middleware
    from django.utils.deprecation import MiddlewareMixin
    MaintenanceModeMiddlewareBaseClass = MiddlewareMixin


class MaintenanceModeMiddleware(MaintenanceModeMiddlewareBaseClass):
    """This middleware will check if the maintenance page should be shown."""

    def process_request(self, request):
        """
        This function will check if maintenance is activated.
        If so return the maintenance page.
        Otherwise continue the request.
        """
        config = MaintenanceMode.get_solo()

        admin_root_url = reverse('admin:index')

        skip_maintenance = False
        if request.path.startswith(admin_root_url) and settings.MAINTENANCE_EXCLUDE_ADMIN_URLS:
            skip_maintenance = True
        if request.user.is_superuser and settings.MAINTENANCE_EXCLUDE_SUPER_USER:
            skip_maintenance = True
        if request.user.is_staff and settings.MAINTENANCE_EXCLUDE_STAFF_USER:
            skip_maintenance = True

        if config.maintenance and not skip_maintenance:
            kwargs = {
                'end_date': config.maintenance_until,
                'display_end_date': settings.MAINTENANCE_DISPLAY_END_DATE
            }

            response = render(
                request, settings.MAINTENANCE_TEMPLATE,
                content_type='text/html', status=503, context=kwargs)
            add_never_cache_headers(response)
            return response
        return None



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-10-26 08:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MaintenanceMode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('maintenance', models.BooleanField(default=False)),
                ('maintenance_from', models.DateTimeField(blank=True, null=True)),
                ('maintenance_until', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'Maintenance Mode',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from solo.models import SingletonModel


@python_2_unicode_compatible
class MaintenanceMode(SingletonModel):
    maintenance = models.BooleanField(default=False)
    maintenance_from = models.DateTimeField(null=True, blank=True)
    maintenance_until = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return u"Maintenance Mode"

    class Meta:
        verbose_name = _("Maintenance Mode")

    def clean(self):
        if self.maintenance_from and not self.maintenance_until:
            raise ValidationError(_('You can not set "maintenance_from" \
                " without setting "maintenance_until"'))
